@@ -36,7 +36,7 @@
 
 #include "udisksdaemon.h"
 #include "udisksdaemonutil.h"
-#include "udiskscleanup.h"
+#include "udisksstate.h"
 #include "udiskslogging.h"
 #include "udiskslinuxblockobject.h"
 #include "udiskslinuxdriveobject.h"
@@ -44,6 +44,8 @@
 #if defined(HAVE_LIBSYSTEMD_LOGIN)
 #include <systemd/sd-daemon.h>
 #include <systemd/sd-login.h>
+
+#define LOGIND_AVAILABLE() (access("/run/systemd/seats/", F_OK) >= 0)
 #endif
 
 /**
@@ -341,20 +343,20 @@ udisks_daemon_util_setup_by_user (UDisksDaemon *daemon,
   gboolean ret;
   UDisksBlock *block = NULL;
   UDisksPartition *partition = NULL;
-  UDisksCleanup *cleanup;
+  UDisksState *state;
   uid_t setup_by_user;
   UDisksObject *crypto_object;
 
   ret = FALSE;
 
-  cleanup = udisks_daemon_get_cleanup (daemon);
+  state = udisks_daemon_get_state (daemon);
   block = udisks_object_get_block (object);
   if (block == NULL)
     goto out;
   partition = udisks_object_get_partition (object);
 
   /* loop devices */
-  if (udisks_cleanup_has_loop (cleanup, udisks_block_get_device (block), &setup_by_user))
+  if (udisks_state_has_loop (state, udisks_block_get_device (block), &setup_by_user))
     {
       if (setup_by_user == user)
         {
@@ -386,9 +388,9 @@ udisks_daemon_util_setup_by_user (UDisksDaemon *daemon,
     {
       UDisksBlock *crypto_block;
       crypto_block = udisks_object_peek_block (crypto_object);
-      if (udisks_cleanup_find_unlocked_luks (cleanup,
-                                             udisks_block_get_device_number (crypto_block),
-                                             &setup_by_user))
+      if (udisks_state_find_unlocked_luks (state,
+                                           udisks_block_get_device_number (crypto_block),
+                                           &setup_by_user))
         {
           if (setup_by_user == user)
             {
@@ -404,7 +406,7 @@ udisks_daemon_util_setup_by_user (UDisksDaemon *daemon,
   if (g_strcmp0 (udisks_block_get_mdraid (block), "/") != 0)
     {
       uid_t started_by_user;
-      if (udisks_cleanup_has_mdraid (cleanup, udisks_block_get_device_number (block), &started_by_user))
+      if (udisks_state_has_mdraid (state, udisks_block_get_device_number (block), &started_by_user))
         {
           if (started_by_user == user)
             {
@@ -635,7 +637,31 @@ udisks_daemon_util_check_authorization_sync (UDisksDaemon          *daemon,
       _safe_polkit_details_insert (details, "drive.model", udisks_drive_get_model (drive));
       _safe_polkit_details_insert (details, "drive.revision", udisks_drive_get_revision (drive));
       if (udisks_drive_get_removable (drive))
-        polkit_details_insert (details, "drive.removable", "true");
+        {
+          const gchar *const *media_compat;
+          GString *media_compat_str;
+          const gchar *sep = ",";
+
+          polkit_details_insert (details, "drive.removable", "true");
+          _safe_polkit_details_insert (details, "drive.removable.bus", udisks_drive_get_connection_bus (drive));
+
+          media_compat_str = g_string_new (NULL);
+          media_compat = udisks_drive_get_media_compatibility (drive);
+          if (media_compat)
+            {
+              guint i;
+
+              for (i = 0; media_compat[i] && strlen(media_compat[i]); i++)
+                {
+                  if (i)
+                    g_string_append (media_compat_str, sep);
+                  g_string_append (media_compat_str, media_compat[i]);
+                }
+            }
+
+          _safe_polkit_details_insert (details, "drive.removable.media", media_compat_str->str);
+          g_string_free (media_compat_str, TRUE);
+        }
     }
 
   if (block != NULL)
@@ -1039,8 +1065,8 @@ udisks_daemon_util_on_same_seat (UDisksDaemon          *daemon,
   UDisksObject *drive_object = NULL;
   UDisksDrive *drive = NULL;
 
-  /* if we haven't booted with systemd, assume it's always the same seat */
-  if (sd_booted () <= 0)
+  /* if we don't have logind, assume it's always the same seat */
+  if (!LOGIND_AVAILABLE())
     return TRUE;
 
   if (UDISKS_IS_LINUX_BLOCK_OBJECT (object))
