@@ -87,6 +87,13 @@ gboolean udisks_daemon_util_file_set_contents (const gchar  *filename,
 UDisksInhibitCookie *udisks_daemon_util_inhibit_system_sync   (const gchar          *reason);
 void                 udisks_daemon_util_uninhibit_system_sync (UDisksInhibitCookie  *cookie);
 
+gboolean udisks_daemon_util_on_same_seat (UDisksDaemon          *daemon,
+                                          UDisksObject          *object,
+                                          pid_t                  process);
+
+gchar *udisks_daemon_util_get_free_mdraid_device (void);
+
+guint16 udisks_ata_identify_get_word (const guchar *identify_data, guint word_number);
 
 G_END_DECLS
 
