@@ -39,7 +39,7 @@
 #include "udiskslinuxblockobject.h"
 #include "udiskslinuxfsinfo.h"
 #include "udisksdaemon.h"
-#include "udiskscleanup.h"
+#include "udisksstate.h"
 #include "udisksdaemonutil.h"
 #include "udisksmountmonitor.h"
 #include "udisksmount.h"
@@ -302,6 +302,13 @@ static const gchar *udf_allow[] = { "iocharset=", "umask=", NULL };
 static const gchar *udf_allow_uid_self[] = { "uid=", NULL };
 static const gchar *udf_allow_gid_self[] = { "gid=", NULL };
 
+/* ---------------------- exfat -------------------- */
+
+static const gchar *exfat_defaults[] = { "uid=", "gid=", "iocharset=utf8", "namecase=0", "errors=remount-ro", "umask=0077", NULL };
+static const gchar *exfat_allow[] = { "dmask=", "errors=", "fmask=", "iocharset=", "namecase=", "umask=", NULL };
+static const gchar *exfat_allow_uid_self[] = { "uid=", NULL };
+static const gchar *exfat_allow_gid_self[] = { "gid=", NULL };
+
 /* ------------------------------------------------ */
 /* TODO: support context= */
 
@@ -313,6 +320,7 @@ static const FSMountOptions fs_mount_options[] =
     { "ntfs", ntfs_defaults, ntfs_allow, ntfs_allow_uid_self, ntfs_allow_gid_self },
     { "iso9660", iso9660_defaults, iso9660_allow, iso9660_allow_uid_self, iso9660_allow_gid_self },
     { "udf", udf_defaults, udf_allow, udf_allow_uid_self, udf_allow_gid_self },
+    { "exfat", exfat_defaults, exfat_allow, exfat_allow_uid_self, exfat_allow_gid_self },
   };
 
 /* ------------------------------------------------ */
@@ -841,6 +849,8 @@ calculate_mount_point (UDisksDaemon              *daemon,
                        const gchar               *fs_type,
                        GError                   **error)
 {
+  UDisksLinuxBlockObject *object = NULL;
+  gboolean fs_shared = FALSE;
   const gchar *label = NULL;
   const gchar *uuid = NULL;
   gchar *escaped_user_name = NULL;
@@ -859,10 +869,25 @@ calculate_mount_point (UDisksDaemon              *daemon,
       uuid = udisks_block_get_id_uuid (block);
     }
 
+  object = udisks_daemon_util_dup_object (block, NULL);
+  if (object != NULL)
+    {
+      UDisksLinuxDevice *device = udisks_linux_block_object_get_device (object);
+      if (device != NULL)
+        {
+          if (device->udev_device != NULL)
+            {
+              /* TODO: maybe introduce Block:HintFilesystemShared instead of pulling it directly from the udev device */
+              fs_shared = g_udev_device_get_property_as_boolean (device->udev_device, "UDISKS_FILESYSTEM_SHARED");
+            }
+          g_object_unref (device);
+        }
+    }
+
   /* If we know the user-name and it doesn't have any '/' character in
    * it, mount in /run/media/$USER
    */
-  if (user_name != NULL && strstr (user_name, "/") == NULL)
+  if (!fs_shared && (user_name != NULL && strstr (user_name, "/") == NULL))
     {
       mount_dir = g_strdup_printf ("/run/media/%s", user_name);
       if (!g_file_test (mount_dir, G_FILE_TEST_EXISTS))
@@ -963,6 +988,7 @@ calculate_mount_point (UDisksDaemon              *daemon,
   g_free (mount_dir);
 
  out:
+  g_clear_object (&object);
   g_free (escaped_user_name);
   return mount_point;
 }
@@ -1103,7 +1129,7 @@ handle_mount (UDisksFilesystem       *filesystem,
   UDisksObject *object;
   UDisksBlock *block;
   UDisksDaemon *daemon;
-  UDisksCleanup *cleanup;
+  UDisksState *state;
   uid_t caller_uid;
   gid_t caller_gid;
   pid_t caller_pid;
@@ -1149,7 +1175,7 @@ handle_mount (UDisksFilesystem       *filesystem,
 
   block = udisks_object_peek_block (object);
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
-  cleanup = udisks_daemon_get_cleanup (daemon);
+  state = udisks_daemon_get_state (daemon);
 
   /* check if mount point is managed by e.g. /etc/fstab or similar */
   if (is_system_managed (block, &mount_point_to_use, &fstab_mount_options))
@@ -1310,11 +1336,11 @@ handle_mount (UDisksFilesystem       *filesystem,
                      caller_uid);
 
       /* update the mounted-fs file */
-      udisks_cleanup_add_mounted_fs (cleanup,
-                                     mount_point_to_use,
-                                     udisks_block_get_device_number (block),
-                                     caller_uid,
-                                     TRUE); /* fstab_mounted */
+      udisks_state_add_mounted_fs (state,
+                                   mount_point_to_use,
+                                   udisks_block_get_device_number (block),
+                                   caller_uid,
+                                   TRUE); /* fstab_mounted */
 
       udisks_filesystem_complete_mount (filesystem, invocation, mount_point_to_use);
       goto out;
@@ -1465,11 +1491,11 @@ handle_mount (UDisksFilesystem       *filesystem,
     }
 
   /* update the mounted-fs file */
-  udisks_cleanup_add_mounted_fs (cleanup,
-                                 mount_point_to_use,
-                                 udisks_block_get_device_number (block),
-                                 caller_uid,
-                                 FALSE); /* fstab_mounted */
+  udisks_state_add_mounted_fs (state,
+                               mount_point_to_use,
+                               udisks_block_get_device_number (block),
+                               caller_uid,
+                               FALSE); /* fstab_mounted */
 
   udisks_notice ("Mounted %s at %s on behalf of uid %d",
                  udisks_block_get_device (block),
@@ -1519,7 +1545,7 @@ handle_unmount (UDisksFilesystem       *filesystem,
   UDisksObject *object;
   UDisksBlock *block;
   UDisksDaemon *daemon;
-  UDisksCleanup *cleanup;
+  UDisksState *state;
   gchar *mount_point;
   gchar *fstab_mount_options;
   gchar *escaped_mount_point;
@@ -1554,7 +1580,7 @@ handle_unmount (UDisksFilesystem       *filesystem,
 
   block = udisks_object_peek_block (object);
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
-  cleanup = udisks_daemon_get_cleanup (daemon);
+  state = udisks_daemon_get_state (daemon);
   system_managed = FALSE;
 
   if (options != NULL)
@@ -1660,10 +1686,10 @@ handle_unmount (UDisksFilesystem       *filesystem,
     }
 
   error = NULL;
-  mount_point = udisks_cleanup_find_mounted_fs (cleanup,
-                                                udisks_block_get_device_number (block),
-                                                &mounted_by_uid,
-                                                &fstab_mounted);
+  mount_point = udisks_state_find_mounted_fs (state,
+                                              udisks_block_get_device_number (block),
+                                              &mounted_by_uid,
+                                              &fstab_mounted);
   if (mount_point == NULL)
     {
       /* allow unmounting stuff not mentioned in mounted-fs, but treat it like root mounted it */
@@ -1741,7 +1767,7 @@ handle_unmount (UDisksFilesystem       *filesystem,
       goto out;
     }
 
-  /* OK, filesystem unmounted.. the cleanup routines will remove the mountpoint for us */
+  /* OK, filesystem unmounted.. the state/cleanup routines will remove the mountpoint for us */
 
   udisks_notice ("Unmounted %s on behalf of uid %d",
                  udisks_block_get_device (block),
@@ -1801,6 +1827,7 @@ handle_set_label (UDisksFilesystem       *filesystem,
   UDisksBaseJob *job;
   const gchar *action_id;
   const gchar *message;
+  gchar *real_label = NULL;
   uid_t caller_uid;
   gid_t caller_gid;
   pid_t caller_pid;
@@ -1875,8 +1902,9 @@ handle_set_label (UDisksFilesystem       *filesystem,
       goto out;
     }
 
-  /* VFAT does not allow some characters; as mlabel hangs with interactive
-   * question in this case, check in advance */
+  /* VFAT does not allow some characters; as dosfslabel does not enforce this,
+   * check in advance; also, VFAT only knows upper-case characters, dosfslabel
+   * enforces this */
   if (g_strcmp0 (probed_fs_type, "vfat") == 0)
     {
       const gchar *forbidden = "\"*/:<>?\\|";
@@ -1893,6 +1921,11 @@ handle_set_label (UDisksFilesystem       *filesystem,
                goto out;
             }
         }
+
+      /* we need to remember that we make a copy, so assign it to a new
+       * variable, too */
+      real_label = g_ascii_strup (label, -1);
+      label = real_label;
     }
 
   /* Fail if the device is already mounted and the tools/drivers doesn't
@@ -1970,6 +2003,8 @@ handle_set_label (UDisksFilesystem       *filesystem,
                     invocation);
 
  out:
+  /* for some FSes we need to copy and modify label; free our copy */
+  g_free (real_label);
   g_free (command);
   g_clear_object (&object);
   return TRUE; /* returning TRUE means that we handled the method invocation */
